@@ -1,4 +1,6 @@
-
+import scConnect as cn
+version = cn.database.version
+organism = cn.database.organism
 
 def build_graph(edge_list, node_list=False):
     """Construct a multi directional graph from an edgelist and nodelist and save as graph.
@@ -164,6 +166,27 @@ def score(G):
 # The flatten graph aproach has been depreceated.
 # Use conversion to adjacency matrix on filtered graphs insted
 
+# filter the graph based on interactions containing ligands and receptors under a threshold p-value
+def significant_interactions(G, a=0.05):
+    """Return a filtered graph containing only sigificanly differentiated ligands and receptor interaction.
+    
+    a: alpha, selects scores with a pvalue under alpha.
+    Return a MultiDiGraph
+    """
+
+    import networkx as nx
+    G_filtered = nx.MultiDiGraph()
+    for u, v, n, d in G.edges(data=True, keys=True):
+        # Find interactions with significant p-values (pval < a)
+        # Must be significantly upregulated (zscore > 0)
+        if d["ligand_pval"] < a and d["receptor_pval"] < a and d["ligand_zscore"] > 0 and d["receptor_zscore"] > 0:
+            G_filtered.add_edge(u, v , key=n ,**d)
+            
+    G_filtered.add_nodes_from(G.nodes(data=True))
+    
+    return G_filtered
+    
+
 
 def flatten_graph(G, weight="score", log=True):
     """
@@ -540,7 +563,13 @@ def compare_interactions_df(G, node_a=str, node_b=str, method="ratio"):
                 a = df[node_a][node]
                 b = df[node_b][node]
                 ratio = a - b  # Calulate the difference in interaction score
-                
+
+            if method == "specificity":
+                df = nx.to_pandas_adjacency(Gs[interaction], weight="specificity")
+                a = df[node_a][node]
+                b = df[node_b][node]
+                ratio = a - b  # Calulate the difference in specificity
+
             ratio_dict[interaction] = ratio
         interaction_dict_i[node] = ratio_dict
     df_i = pd.DataFrame(interaction_dict_i)
@@ -586,7 +615,7 @@ def compare_interactions_plot(G, node_a=str, node_b=str, th=float, figsize_i=(5,
     import scConnect as cn
     import matplotlib.pyplot as plt
     
-    df_i, df_o = compare_interactions_df(G, node_a, node_b)
+    df_i, df_o = compare_interactions_df(G, node_a, node_b, method)
 
     if method == "ratio": # convert a ratio th to the corresponding log values
         th = np.log10(th)
@@ -612,36 +641,38 @@ def compare_interactions_plot(G, node_a=str, node_b=str, th=float, figsize_i=(5,
     unique_families = set(families_dict.values())
     lut = dict(zip(unique_families, sns.color_palette(colormap, len(unique_families))))
 
-    # Make color mapping for incomming interactions
-    row_colors_i = list()
-    for receptor in receptors_i:
-        family = families_dict[receptor]
-        color = lut[family]
-        row_colors_i.append(color)
-
-    # plot a clustered heatmap with receptor families as cluster row color for incomming interactions
-    print("incomming interactions")
+    if df_i.shape[0] > 1:
+        # Make color mapping for incomming interactions
+        row_colors_i = list()
+        for receptor in receptors_i:
+            family = families_dict[receptor]
+            color = lut[family]
+            row_colors_i.append(color)
 
+        # plot a clustered heatmap with receptor families as cluster row color for incomming interactions
+        print("incomming interactions")
 
     
-    sns.clustermap(df_i, center=0, cmap="bwr", figsize=figsize_i, row_colors=row_colors_i)
-    if save != None:
-        path_i = f"{save}{node_a}_to_{node_b}_incomming.pdf"
-        plt.savefig(path_i)
+    
+        sns.clustermap(df_i, center=0, cmap="bwr", figsize=figsize_i, row_colors=row_colors_i)
+        if save != None:
+            path_i = f"{save}{node_a}_to_{node_b}_incomming.pdf"
+            plt.savefig(path_i)
 
-     # Make color mapping for outgoing interactions
-    row_colors_o = list()
-    for receptor in receptors_o:
-        family = families_dict[receptor]
-        color = lut[family]
-        row_colors_o.append(color)
+    if df_o.shape[0] > 1:
+        # Make color mapping for outgoing interactions
+        row_colors_o = list()
+        for receptor in receptors_o:
+            family = families_dict[receptor]
+            color = lut[family]
+            row_colors_o.append(color)
 
-    # plot a clustered heatmap with receptor families as cluster row color
-    print("Outgoing interactions")
-    sns.clustermap(df_o, center=0, cmap="bwr", figsize=figsize_o, row_colors=row_colors_o)
-    if save != None:
-        path_o = f"{save}{node_a}_to_{node_b}_outgoing.pdf"
-        plt.savefig(path_o)
+        # plot a clustered heatmap with receptor families as cluster row color
+        print("Outgoing interactions")
+        sns.clustermap(df_o, center=0, cmap="bwr", figsize=figsize_o, row_colors=row_colors_o)
+        if save != None:
+            path_o = f"{save}{node_a}_to_{node_b}_outgoing.pdf"
+            plt.savefig(path_o)
 
     print(f"receptor families in order: {list(lut.keys())}")
     sns.palplot(lut.values())
@@ -702,3 +733,190 @@ def plot_interaction_heatmap(G, th=0.5, method='ward', metric='euclidean', z_sco
     f.savefig(save)
     
     return f
+
+
+# Fetch edge list dataframe for plotting functions
+def edge_list(
+    G, 
+    filter_by="specificity", 
+    th=1, 
+    cellphonedb_interactions=False, 
+    interaction_list=None,
+    version = version,
+    organism = organism,
+    all_connections=True):
+    """retrieves all interactions from the graphs and returns a pandas dataframe suitable for plotting"""
+
+    import networkx as nx
+    import pandas as pd
+    import numpy as np
+    import plotly.express as px
+    import pkg_resources
+    
+    
+    df = nx.to_pandas_edgelist(G)
+    
+    # change name of scConnects interaction to better compare to cellPhoneDB
+    receptor_info = pd.read_csv(pkg_resources.resource_filename("scConnect", (f"data/Gene_annotation/{version}/{organism}/receptors.csv")), index_col=1)
+    ligand_info = pd.read_csv(pkg_resources.resource_filename("scConnect", (f"data/Gene_annotation/{version}/{organism}/ligands.csv")), index_col=1)
+
+    df["ligand_gene"] = [eval(ligand_info.loc[ligand]["preprogene"])[0] if isinstance(ligand_info.loc[ligand]["preprogene"], str) else ligand for ligand in df["ligand"]]
+    df["receptor_gene"] = [eval(receptor_info.loc[receptor]["gene"])[0] if isinstance(receptor_info.loc[receptor]["gene"], str) else receptor for receptor in df["receptor"]]
+
+    df["cellphonedb_interactions"] = [f"{l}_{r}" for l, r in list(zip(df["ligand_gene"], df["receptor_gene"]))]
+
+    #Create a new connection annotation merging source and target info
+    df["connection"] = [str(connection[0]+"|"+connection[1]) for connection in zip(df["source"], df["target"])]
+
+    #Collect all connection for specific interactions
+    if all_connections:
+        # select all significant interaction (significance > 1)
+        interactions = df[df[filter_by] >= th]["interaction"].unique()
+        # keep all interactions that were significant between any connection
+        df = df[[True if interaction in interactions else False for interaction in df["interaction"]]]
+
+        # only sort if all_connections == True
+         # sort based on connection and interactoin names to get order in the plot
+        if cellphonedb_interactions:
+            df.sort_values(by=[ "cellphonedb_interactions", "connection"], key=np.vectorize(str.lower), ascending=False, inplace=True)
+        else:
+            df.sort_values(by=["connection", "interaction"], key=np.vectorize(str.lower), ascending=False, inplace=True)
+    
+    else:
+        # if only selected interactions, sort by filter_by key
+        df= df[df[filter_by] >= th]
+        df.sort_values(by=[filter_by], ascending=False, inplace=True)
+   
+    return df
+
+# plotting function using the edge list data
+def dotplot(
+    G=None, 
+    df=None, 
+    filter_by="specificity", 
+    th=1,
+    organism=organism,
+    version=version,
+    all_connections=True,
+    cellphonedb_interactions=False, 
+    height_scale=40, 
+    width_scale=40, 
+    cmap=None):
+
+    import plotly.express as px
+    import pandas as pd
+    import numpy as np
+
+    if cmap == None:
+        cmap = px.colors.sequential.Viridis_r
+    
+    if isinstance(df, pd.DataFrame):
+        df = df
+        # Filter by feature and threshold
+        interactions = df[df[filter_by] >= th]["interaction"].unique()
+        # keep all interactions that were significant between any connection
+        df = df[[True if interaction in interactions else False for interaction in df["interaction"]]]
+    else:
+        df = edge_list(G, filter_by, th, organism=organism, version=version, all_connections=all_connections)
+        
+    # set width and hight of figure
+    width = len(df["connection"].unique())*width_scale
+    height = len(df["interaction"].unique())*height_scale
+    if cellphonedb_interactions:
+        y = "cellphonedb_interactions"
+    else:
+        y = "interaction"
+    
+    # log (specificity +1)for plotting sized
+    df["log_specificity"] = np.log10(df["specificity"]+1)
+    
+    # find and sort all connections
+    connections = list(set(df["connection"]))
+    connections.sort()
+
+    
+    
+    plot = px.scatter(df, x="connection", y=y, 
+        size="log_specificity", 
+        color="log_score",
+        opacity=0.8,
+        height=height, width=width,
+        size_max=10,
+        template="plotly_white",
+        render_mode="svg",
+        title=f"Interactions with {filter_by} higher than {th}",
+        color_continuous_scale=cmap,
+        hover_data=["specificity"],
+        category_orders = {"connection": connections})
+    
+    plot.update_layout(coloraxis_colorbar=dict(
+        title="Log score",
+        thicknessmode="pixels", thickness = 20,
+        lenmode="pixels", len=200,
+        yanchor="top", y=1
+        )
+    )
+    return plot
+    
+
+# functions to retrieve dataframes of ligand and receptor
+# information for further analysis
+def get_ligand_df(G, color_map=False, corr_pval=True):
+    """fetch all ligand information in a graph and return a DataFrame in long format"""
+    import pandas as pd
+    import numpy as np
+
+    nodes = G.nodes(data=True)
+    df = pd.DataFrame(columns=["ligand", "score", "node"])
+    for node, data in G.nodes.items():
+        temp_df = pd.DataFrame.from_dict(data["ligands_score"], orient="index", columns=["score"])
+        temp_df["z_score"] = pd.DataFrame.from_dict(data["ligands_zscore"], orient="index", columns=["z-score"])
+        if corr_pval:
+            temp_df["pvalue"] = pd.DataFrame.from_dict(data["ligands_corr_pval"], orient="index", columns=["pvalue"])
+        else:
+             temp_df["pvalue"] = pd.DataFrame.from_dict(data["ligands_pval"], orient="index", columns=["pvalue"])
+        temp_df["specificity"] = -np.log10(temp_df["pvalue"])
+        temp_df["log_score"] = np.log10(temp_df["score"]+1)
+        temp_df.rename_axis(index="ligand", inplace=True)
+        temp_df["node"] = node
+        temp_df.reset_index(inplace=True)
+        df = df.append(temp_df, ignore_index=True,)
+        
+    color = dict(G.nodes(data="color"))
+    for k, v in color.items():
+        color[k] = "rgb" + str(tuple(v))
+        
+    if color_map == True:
+        return df, color
+    
+    return df
+
+def get_receptor_df(G, color_map=False, corr_pval=True):
+    """fetch all ligand information in a graph and return a DataFrame in long format"""
+    import pandas as pd
+    import numpy as np
+
+    nodes = G.nodes(data=True)
+    df = pd.DataFrame(columns=["receptor", "score", "node"])
+    for node, data in G.nodes.items():
+        temp_df = pd.DataFrame.from_dict(data["receptors_score"], orient="index", columns=["score"])
+        temp_df["z_score"] = pd.DataFrame.from_dict(data["receptors_zscore"], orient="index", columns=["z-score"])
+        if corr_pval:
+            temp_df["pvalue"] = pd.DataFrame.from_dict(data["receptors_corr_pval"], orient="index", columns=["pvalue"])
+        else:
+            temp_df["pvalue"] = pd.DataFrame.from_dict(data["receptors_pval"], orient="index", columns=["pvalue"])
+        temp_df["specificity"] = -np.log10(temp_df["pvalue"])
+        temp_df["log_score"] = np.log10(temp_df["score"]+1)
+        temp_df.rename_axis(index="receptor", inplace=True)
+        temp_df["node"] = node
+        temp_df.reset_index(inplace=True)
+        df = df.append(temp_df, ignore_index=True,)
+        
+    color = dict(G.nodes(data="color"))
+    for k, v in color.items():
+        color[k] = "rgb" + str(tuple(v))
+        
+    if color_map == True:
+        return df, color
+    
+    return df
\ No newline at end of file
